#!/usr/bin/env python
#
# Copyright (c) 2001, 2002, 2003, 2004 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

__revision__ = "/home/scons/scons/branch.0/baseline/test/Repository/M4.py 0.96.1.D001 2004/08/23 09:55:29 knight"

"""
Test that $M4 and $M4FLAGS work with repositories.
"""

import os
import os.path
import string
import sys
import TestSCons

python = TestSCons.python

test = TestSCons.TestSCons()

test.subdir('work', 'repository', ['repository', 'src'])

test.write('mym4.py', """
import string
import sys
contents = sys.stdin.read()
sys.stdout.write(string.replace(contents, 'M4', 'mym4.py'))
sys.exit(0)
""")




opts = "-Y " + test.workpath('repository')

test.write(['repository', 'SConstruct'], """\
env = Environment(M4 = r'%s %s', tools=['default', 'm4'])
env.M4(target = 'aaa.x', source = 'aaa.x.m4')
SConscript('src/SConscript', "env", build_dir="build")
""" % (python, test.workpath('mym4.py')))

test.write(['repository', 'aaa.x.m4'], """\
line 1
M4
line 3
""")

test.write(['repository', 'src', 'SConscript'], """
Import("env")
env.M4('bbb.y', 'bbb.y.m4')
""")

test.write(['repository', 'src', 'bbb.y.m4'], """\
line 1 M4
line 2
line 3 M4
""")

#
# Make the repository non-writable,
# so we'll detect if we try to write into it accidentally.
test.writable('repository', 0)

#
test.run(chdir = 'work', options = opts, arguments = ".")

expect_aaa_x = """\
line 1
mym4.py
line 3
"""

expect_bbb_y = """\
line 1 mym4.py
line 2
line 3 mym4.py
"""

test.fail_test(test.read(test.workpath('work', 'aaa.x'), 'r') != expect_aaa_x)
test.fail_test(test.read(test.workpath('work', 'build', 'bbb.y'), 'r') != expect_bbb_y)

#
test.pass_test()
