#!/usr/bin/env python
#
# Copyright (c) 2001, 2002, 2003, 2004 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

__revision__ = "/home/scons/scons/branch.0/baseline/test/SetBuildSignatureType.py 0.96.1.D001 2004/08/23 09:55:29 knight"

import TestSCons

test = TestSCons.TestSCons()

warning = """\

scons: warning: The SetBuildSignatureType() function has been deprecated;
	use the TargetSignatures() function instead.
File "SConstruct", line %d, in ?
"""

test.write('SConstruct', """
env = Environment()

def copy1(env, source, target):
    open(str(target[0]), 'wb').write(open(str(source[0]), 'rb').read())

def copy2(env, source, target):
    return copy1(env, source, target)

env['BUILDERS']['Copy1'] = Builder(action=copy1)
env['BUILDERS']['Copy2'] = Builder(action=copy2)

env.Copy2('foo.out', 'foo.in')
env.Copy1('foo.out.out', 'foo.out')

SetBuildSignatureType('content')
""")

test.write('foo.in', 'foo.in')

test.run(arguments='foo.out.out',
         stdout=test.wrap_stdout("""\
copy2(["foo.out"], ["foo.in"])
copy1(["foo.out.out"], ["foo.out"])
"""),
         stderr=warning%16)

test.run(arguments='foo.out.out',
         stdout=test.wrap_stdout("scons: `foo.out.out' is up to date.\n"),
         stderr=warning%16)

test.write('SConstruct', """
env = Environment()

def copy1(env, source, target):
    open(str(target[0]), 'wb').write(open(str(source[0]), 'rb').read())

def copy2(env, source, target):
    x = 2 # added this line
    return copy1(env, source, target)

env['BUILDERS']['Copy1'] = Builder(action=copy1)
env['BUILDERS']['Copy2'] = Builder(action=copy2)

env.Copy2('foo.out', 'foo.in')
env.Copy1('foo.out.out', 'foo.out')

SetBuildSignatureType('content')
""")

test.run(arguments='foo.out.out',
         stdout=test.wrap_stdout("""\
copy2(["foo.out"], ["foo.in"])
scons: `foo.out.out' is up to date.
"""),
         stderr=warning%17)

test.write('SConstruct', """
env = Environment()

def copy1(env, source, target):
    open(str(target[0]), 'wb').write(open(str(source[0]), 'rb').read())

def copy2(env, source, target):
    x = 2 # added this line
    return copy1(env, source, target)

env['BUILDERS']['Copy1'] = Builder(action=copy1)
env['BUILDERS']['Copy2'] = Builder(action=copy2)

env.Copy2('foo.out', 'foo.in')
env.Copy1('foo.out.out', 'foo.out')

SetBuildSignatureType('build')
""")

test.run(arguments='foo.out.out',
         stdout=test.wrap_stdout("""\
copy1(["foo.out.out"], ["foo.out"])
"""),
         stderr=warning%17)

test.write('SConstruct', """
env = Environment()

def copy1(env, source, target):
    open(str(target[0]), 'wb').write(open(str(source[0]), 'rb').read())

def copy2(env, source, target):
    return copy1(env, source, target)

env['BUILDERS']['Copy1'] = Builder(action=copy1)
env['BUILDERS']['Copy2'] = Builder(action=copy2)

env.Copy2('foo.out', 'foo.in')
env.Copy1('foo.out.out', 'foo.out')

SetBuildSignatureType('build')
""")

test.run(arguments='foo.out.out',
         stdout=test.wrap_stdout("""\
copy2(["foo.out"], ["foo.in"])
copy1(["foo.out.out"], ["foo.out"])
"""),
         stderr=warning%16)


test.pass_test()
